@@ -2,14 +2,13 @@
 
 use std::fmt::Write;
 
-use tracing::{instrument, warn};
+use eden_dag::DagAlgorithm;
+use tracing::instrument;
 
 use crate::commands::smartlog::smartlog;
 use crate::core::eventlog::{EventLogDb, EventReplayer};
 use crate::core::formatting::printable_styled_string;
-use crate::core::graph::{make_graph, BranchOids, CommitGraph, HeadOid, MainBranchOid};
-use crate::core::mergebase::{make_merge_base_db, MergeBaseDb};
-use crate::git::{GitRunInfo, NonZeroOid, Repo};
+use crate::git::{sort_commit_set, CommitSet, Dag, GitRunInfo, NonZeroOid, Repo};
 use crate::tui::Effects;
 
 /// Go back a certain number of commits.
@@ -30,7 +29,7 @@ pub fn prev(
     if exit_code != 0 {
         return Ok(exit_code);
     }
-    smartlog(effects)?;
+    smartlog(effects, &Default::default())?;
     Ok(0)
 }
 
@@ -47,41 +46,10 @@ pub enum Towards {
 }
 
 #[instrument]
-fn advance_towards_main_branch(
+fn advance(
     effects: &Effects,
     repo: &Repo,
-    merge_base_db: &impl MergeBaseDb,
-    graph: &CommitGraph,
-    current_oid: NonZeroOid,
-    main_branch_oid: &MainBranchOid,
-) -> eyre::Result<(isize, NonZeroOid)> {
-    let MainBranchOid(main_branch_oid) = main_branch_oid;
-    let path =
-        merge_base_db.find_path_to_merge_base(effects, repo, *main_branch_oid, current_oid)?;
-    let path = match path {
-        None => return Ok((0, current_oid)),
-        Some(path) if path.len() == 1 => {
-            // Must be the case that `current_oid == main_branch_oid`.
-            return Ok((0, current_oid));
-        }
-        Some(path) => path,
-    };
-
-    for (i, commit) in (1..).zip(path.iter().rev().skip(1)) {
-        if graph.contains_key(&commit.get_oid()) {
-            return Ok((i, commit.get_oid()));
-        }
-    }
-
-    warn!("Failed to find graph commit when advancing towards main branch");
-    Ok((0, current_oid))
-}
-
-#[instrument]
-fn advance_towards_own_commit(
-    effects: &Effects,
-    repo: &Repo,
-    graph: &CommitGraph,
+    dag: &Dag,
     current_oid: NonZeroOid,
     num_commits: isize,
     towards: Option<Towards>,
@@ -89,16 +57,21 @@ fn advance_towards_own_commit(
     let glyphs = effects.get_glyphs();
     let mut current_oid = current_oid;
     for i in 0..num_commits {
-        let children = &graph[&current_oid].children;
+        let children = dag
+            .query()
+            .children(CommitSet::from(current_oid))?
+            .difference(&dag.obsolete_commits);
+        let children = sort_commit_set(repo, dag, &children)?;
+
         current_oid = match (towards, children.as_slice()) {
             (_, []) => {
                 // It would also make sense to issue an error here, rather than
                 // silently stop going forward commits.
                 break;
             }
-            (_, [only_child_oid]) => *only_child_oid,
-            (Some(Towards::Newest), [.., newest_child_oid]) => *newest_child_oid,
-            (Some(Towards::Oldest), [oldest_child_oid, ..]) => *oldest_child_oid,
+            (_, [only_child]) => only_child.get_oid(),
+            (Some(Towards::Newest), [.., newest_child]) => newest_child.get_oid(),
+            (Some(Towards::Oldest), [oldest_child, ..]) => oldest_child.get_oid(),
             (None, [_, _, ..]) => {
                 writeln!(
                     effects.get_output_stream(),
@@ -106,7 +79,7 @@ fn advance_towards_own_commit(
                     i
                 )?;
 
-                for (j, child_oid) in (0..).zip(children.iter()) {
+                for (j, child) in (0..).zip(children.iter()) {
                     let descriptor = if j == 0 {
                         " (oldest)"
                     } else if j + 1 == children.len() {
@@ -119,10 +92,7 @@ fn advance_towards_own_commit(
                         effects.get_output_stream(),
                         "  {} {}{}",
                         glyphs.bullet_point,
-                        printable_styled_string(
-                            glyphs,
-                            repo.friendly_describe_commit_from_oid(*child_oid)?
-                        )?,
+                        printable_styled_string(glyphs, child.friendly_describe()?)?,
                         descriptor
                     )?;
                 }
@@ -143,41 +113,28 @@ pub fn next(
     towards: Option<Towards>,
 ) -> eyre::Result<isize> {
     let repo = Repo::from_current_dir()?;
+    let references_snapshot = repo.get_references_snapshot()?;
     let conn = repo.get_db_conn()?;
     let event_log_db = EventLogDb::new(&conn)?;
     let event_replayer = EventReplayer::from_event_log_db(effects, &repo, &event_log_db)?;
-    let merge_base_db = make_merge_base_db(effects, &repo, &conn, &event_replayer)?;
-
-    let head_oid = match repo.get_head_info()?.oid {
-        Some(head_oid) => head_oid,
-        None => eyre::bail!("No HEAD present; cannot calculate next commit"),
-    };
-    let main_branch_oid = repo.get_main_branch_oid()?;
-    let branch_oid_to_names = repo.get_branch_oid_to_names()?;
-    let graph = make_graph(
+    let event_cursor = event_replayer.make_default_cursor();
+    let dag = Dag::open_and_sync(
         effects,
         &repo,
-        &merge_base_db,
         &event_replayer,
-        event_replayer.make_default_cursor(),
-        &HeadOid(Some(head_oid)),
-        &MainBranchOid(main_branch_oid),
-        &BranchOids(branch_oid_to_names.keys().copied().collect()),
-        true,
+        event_cursor,
+        &references_snapshot,
     )?;
 
+    let head_oid = match references_snapshot.head_oid {
+        Some(head_oid) => head_oid,
+        None => {
+            eyre::bail!("No HEAD present; cannot calculate next commit");
+        }
+    };
+
     let num_commits = num_commits.unwrap_or(1);
-    let (num_commits_traversed_towards_main_branch, current_oid) = advance_towards_main_branch(
-        effects,
-        &repo,
-        &merge_base_db,
-        &graph,
-        head_oid,
-        &MainBranchOid(main_branch_oid),
-    )?;
-    let num_commits = num_commits - num_commits_traversed_towards_main_branch;
-    let current_oid =
-        advance_towards_own_commit(effects, &repo, &graph, current_oid, num_commits, towards)?;
+    let current_oid = advance(effects, &repo, &dag, head_oid, num_commits, towards)?;
     let current_oid = match current_oid {
         None => return Ok(1),
         Some(current_oid) => current_oid,
@@ -188,6 +145,6 @@ pub fn next(
         return Ok(result);
     }
 
-    smartlog(effects)?;
+    smartlog(effects, &Default::default())?;
     Ok(0)
 }
