@@ -18,51 +18,38 @@ use cursive::{Cursive, CursiveRunnable, CursiveRunner};
 use eyre::Context;
 use tracing::instrument;
 
-use crate::commands::smartlog::render_graph;
+use crate::commands::smartlog::{make_smartlog_graph, render_graph};
 use crate::core::eventlog::{Event, EventCursor, EventLogDb, EventReplayer, EventTransactionId};
 use crate::core::formatting::{printable_styled_string, Pluralize, StyledStringBuilder};
-use crate::core::graph::{make_graph, BranchOids, HeadOid, MainBranchOid};
-use crate::core::mergebase::{make_merge_base_db, MergeBaseDb};
 use crate::core::metadata::{
     BranchesProvider, CommitMessageProvider, CommitOidProvider, DifferentialRevisionProvider,
-    HiddenExplanationProvider, RelativeTimeProvider,
+    ObsolescenceExplanationProvider, RelativeTimeProvider,
 };
 use crate::declare_views;
-use crate::git::{CategorizedReferenceName, GitRunInfo, MaybeZeroOid, Repo};
+use crate::git::{CategorizedReferenceName, Dag, GitRunInfo, MaybeZeroOid, Repo};
 use crate::tui::{with_siv, Effects, SingletonView};
 
 fn render_cursor_smartlog(
     effects: &Effects,
     repo: &Repo,
-    merge_base_db: &impl MergeBaseDb,
+    dag: &Dag,
     event_replayer: &EventReplayer,
     event_cursor: EventCursor,
 ) -> eyre::Result<Vec<StyledString>> {
-    let head_oid = event_replayer.get_cursor_head_oid(event_cursor);
-    let main_branch_oid = event_replayer.get_cursor_main_branch_oid(event_cursor, repo)?;
-    let branch_oid_to_names = event_replayer.get_cursor_branch_oid_to_names(event_cursor, repo)?;
-    let graph = make_graph(
-        effects,
-        repo,
-        merge_base_db,
-        event_replayer,
-        event_cursor,
-        &HeadOid(head_oid),
-        &MainBranchOid(main_branch_oid),
-        &BranchOids(branch_oid_to_names.keys().copied().collect()),
-        true,
-    )?;
+    let dag = dag.set_cursor(effects, repo, event_replayer, event_cursor)?;
+    let references_snapshot = event_replayer.get_references_snapshot(repo, event_cursor)?;
+    let graph = make_smartlog_graph(effects, repo, &dag, event_replayer, event_cursor, true)?;
     let result = render_graph(
         effects,
         repo,
-        merge_base_db,
+        &dag,
         &graph,
-        &HeadOid(head_oid),
+        references_snapshot.head_oid,
         &mut [
             &mut CommitOidProvider::new(true)?,
             &mut RelativeTimeProvider::new(repo, SystemTime::now())?,
-            &mut HiddenExplanationProvider::new(&graph, event_replayer, event_cursor)?,
-            &mut BranchesProvider::new(repo, &branch_oid_to_names)?,
+            &mut ObsolescenceExplanationProvider::new(event_replayer, event_cursor)?,
+            &mut BranchesProvider::new(repo, &references_snapshot)?,
             &mut DifferentialRevisionProvider::new(repo)?,
             &mut CommitMessageProvider::new()?,
         ],
@@ -90,7 +77,7 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
             ]
         }
 
-        Event::HideEvent {
+        Event::ObsoleteEvent {
             timestamp: _,
             event_tx_id: _,
             commit_oid,
@@ -110,7 +97,7 @@ fn describe_event(repo: &Repo, event: &Event) -> eyre::Result<Vec<StyledString>>
             ]
         }
 
-        Event::UnhideEvent {
+        Event::UnobsoleteEvent {
             timestamp: _,
             event_tx_id: _,
             commit_oid,
@@ -316,7 +303,7 @@ fn select_past_event(
     mut siv: CursiveRunner<CursiveRunnable>,
     effects: &Effects,
     repo: &Repo,
-    merge_base_db: &impl MergeBaseDb,
+    dag: &Dag,
     event_replayer: &mut EventReplayer,
 ) -> eyre::Result<Option<EventCursor>> {
     #[derive(Clone, Copy, Debug)]
@@ -383,7 +370,7 @@ fn select_past_event(
                       event_cursor: EventCursor|
          -> eyre::Result<()> {
             let smartlog =
-                render_cursor_smartlog(effects, repo, merge_base_db, event_replayer, event_cursor)?;
+                render_cursor_smartlog(effects, repo, dag, event_replayer, event_cursor)?;
             SmartlogView::find(siv)
                 .get_inner_mut()
                 .set_content(StyledStringBuilder::from_lines(smartlog));
@@ -543,21 +530,21 @@ fn inverse_event(
             event_tx_id: _,
             commit_oid,
         }
-        | Event::UnhideEvent {
+        | Event::UnobsoleteEvent {
             timestamp: _,
             event_tx_id: _,
             commit_oid,
-        } => Event::HideEvent {
+        } => Event::ObsoleteEvent {
             timestamp,
             event_tx_id,
             commit_oid,
         },
 
-        Event::HideEvent {
+        Event::ObsoleteEvent {
             timestamp: _,
             event_tx_id: _,
             commit_oid,
-        } => Event::UnhideEvent {
+        } => Event::UnobsoleteEvent {
             timestamp,
             event_tx_id,
             commit_oid,
@@ -774,8 +761,8 @@ fn undo_events(
                 repo.create_reference(&ref_name, new_oid, true, "branchless undo")?;
             }
             Event::CommitEvent { .. }
-            | Event::HideEvent { .. }
-            | Event::UnhideEvent { .. }
+            | Event::ObsoleteEvent { .. }
+            | Event::UnobsoleteEvent { .. }
             | Event::RewriteEvent { .. } => {
                 event_log_db.add_events(vec![event])?;
             }
@@ -794,14 +781,27 @@ fn undo_events(
 #[instrument]
 pub fn undo(effects: &Effects, git_run_info: &GitRunInfo) -> eyre::Result<isize> {
     let repo = Repo::from_current_dir()?;
+    let references_snapshot = repo.get_references_snapshot()?;
     let conn = repo.get_db_conn()?;
     let mut event_log_db = EventLogDb::new(&conn)?;
     let mut event_replayer = EventReplayer::from_event_log_db(effects, &repo, &event_log_db)?;
-    let merge_base_db = make_merge_base_db(effects, &repo, &conn, &event_replayer)?;
+    let dag = {
+        // Don't let `event_cursor` leak from this scope, since we intend to
+        // determine a new event cursor below.
+        let event_cursor = event_replayer.make_default_cursor();
+
+        Dag::open_and_sync(
+            effects,
+            &repo,
+            &event_replayer,
+            event_cursor,
+            &references_snapshot,
+        )?
+    };
 
     let event_cursor = {
         let result = with_siv(effects, |effects, siv| {
-            select_past_event(siv, &effects, &repo, &merge_base_db, &mut event_replayer)
+            select_past_event(siv, &effects, &repo, &dag, &mut event_replayer)
         })?;
         match result {
             Some(event_cursor) => event_cursor,
@@ -828,18 +828,17 @@ pub mod testing {
     use cursive::{CursiveRunnable, CursiveRunner};
 
     use crate::core::eventlog::{EventCursor, EventLogDb, EventReplayer};
-    use crate::core::mergebase::MergeBaseDb;
-    use crate::git::{GitRunInfo, Repo};
+    use crate::git::{Dag, GitRunInfo, Repo};
     use crate::tui::Effects;
 
     pub fn select_past_event(
         siv: CursiveRunner<CursiveRunnable>,
         effects: &Effects,
         repo: &Repo,
-        merge_base_db: &impl MergeBaseDb,
+        dag: &Dag,
         event_replayer: &mut EventReplayer,
     ) -> eyre::Result<Option<EventCursor>> {
-        super::select_past_event(siv, effects, repo, merge_base_db, event_replayer)
+        super::select_past_event(siv, effects, repo, dag, event_replayer)
     }
 
     pub fn undo_events(
