@@ -328,6 +328,26 @@ impl Git {
         self.init_repo_with_options(&Default::default())
     }
 
+    /// Clone this repository into the `target` repository (which must not have
+    /// been initialized).
+    pub fn clone_repo_into(&self, target: &Git, additional_args: &[&str]) -> eyre::Result<()> {
+        let args = {
+            let mut args = vec![
+                "clone",
+                // For Windows in CI.
+                "-c",
+                "core.autocrlf=false",
+                self.repo_path.to_str().unwrap(),
+                target.repo_path.to_str().unwrap(),
+            ];
+            args.extend(additional_args.iter());
+            args
+        };
+
+        let (_stdout, _stderr) = self.run(args.as_slice())?;
+        Ok(())
+    }
+
     /// Write the provided contents to the provided file in the repository root.
     pub fn write_file(&self, name: &str, contents: &str) -> eyre::Result<()> {
         let path = PathBuf::from(name);
@@ -487,10 +507,12 @@ fn copy_dir_all(src: impl AsRef<Path>, dst: impl AsRef<Path>) -> std::io::Result
 }
 
 impl GitWrapper {
-    /// Make a copy of the repo on disk. This can be used to reuse testing setup.
+    /// Make a copy of the repo on disk. This can be used to reuse testing
+    /// setup.  This is *not* the same as running `git clone`; it's used to save
+    /// initialization time as part of testing optimization.
     ///
     /// The copied repo will be deleted once the returned value has been dropped.
-    pub fn clone_repo(&self) -> eyre::Result<Self> {
+    pub fn duplicate_repo(&self) -> eyre::Result<Self> {
         let repo_dir = tempfile::tempdir()?;
         copy_dir_all(&self.repo_dir, &repo_dir)?;
         let git = Git {
@@ -513,3 +535,52 @@ pub fn make_git() -> eyre::Result<GitWrapper> {
     let git = Git::new(repo_dir.path().to_path_buf(), path_to_git);
     Ok(GitWrapper { repo_dir, git })
 }
+
+/// Represents a pair of directories that will be cleaned up after this value
+/// dropped. The two directories need to be `init`ed and `clone`ed by the
+/// caller, respectively.
+pub struct GitWrapperWithRemoteRepo {
+    /// Guard to clean up the containing temporary directory. Make sure to bind
+    /// this to a local variable not named `_`.
+    pub temp_dir: TempDir,
+
+    /// The wrapper around the original repository.
+    pub original_repo: Git,
+
+    /// The wrapper around the cloned repository.
+    pub cloned_repo: Git,
+}
+
+/// Create a `GitWrapperWithRemoteRepo`.
+pub fn make_git_with_remote_repo() -> eyre::Result<GitWrapperWithRemoteRepo> {
+    let path_to_git = get_path_to_git()?;
+    let temp_dir = tempfile::tempdir()?;
+    let git_run_info = GitRunInfo {
+        path_to_git,
+        working_directory: temp_dir.path().to_path_buf(),
+        env: Default::default(),
+    };
+    let original_repo_path = temp_dir.path().join("original");
+    std::fs::create_dir_all(&original_repo_path)?;
+    let original_repo = Git::new(
+        original_repo_path.clone(),
+        GitRunInfo {
+            working_directory: original_repo_path,
+            ..git_run_info.clone()
+        },
+    );
+    let cloned_repo_path = temp_dir.path().join("cloned");
+    let cloned_repo = Git::new(
+        cloned_repo_path.clone(),
+        GitRunInfo {
+            working_directory: cloned_repo_path,
+            ..git_run_info
+        },
+    );
+
+    Ok(GitWrapperWithRemoteRepo {
+        temp_dir,
+        original_repo,
+        cloned_repo,
+    })
+}
