@@ -1,4 +1,6 @@
-use branchless::testing::{make_git, GitRunOptions};
+use branchless::testing::{
+    make_git, make_git_with_remote_repo, GitInitOptions, GitRunOptions, GitWrapperWithRemoteRepo,
+};
 
 use crate::command::test_restack::remove_rebase_lines;
 
@@ -21,7 +23,7 @@ fn test_move_stick() -> eyre::Result<()> {
 
     // --on-disk
     {
-        let git = git.clone_repo()?;
+        let git = git.duplicate_repo()?;
         git.run(&[
             "move",
             "--on-disk",
@@ -126,7 +128,7 @@ fn test_move_tree() -> eyre::Result<()> {
 
     // --on-disk
     {
-        let git = git.clone_repo()?;
+        let git = git.duplicate_repo()?;
         git.run(&[
             "move",
             "--on-disk",
@@ -199,7 +201,7 @@ fn test_move_with_source_not_in_smartlog() -> eyre::Result<()> {
 
     // on-disk
     {
-        let git = git.clone_repo()?;
+        let git = git.duplicate_repo()?;
 
         git.run(&[
             "move",
@@ -303,12 +305,29 @@ fn test_move_merge_conflict() -> eyre::Result<()> {
     git.run(&["checkout", &base_oid.to_string()])?;
     git.commit_file_with_contents("conflict", 2, "conflict 2\n")?;
 
+    {
+        let (stdout, _stderr) = git.run_with_options(
+            &["move", "--source", &other_oid.to_string()],
+            &GitRunOptions {
+                expected_exit_code: 1,
+                ..Default::default()
+            },
+        )?;
+        insta::assert_snapshot!(stdout, @r###"
+        Attempting rebase in-memory...
+        This operation would cause a merge conflict:
+        - (1 conflicting file) e85d25c7 create conflict.txt
+        To resolve merge conflicts, retry this operation with the --merge option.
+        "###);
+    }
+
     {
         let (stdout, _stderr) = git.run_with_options(
             &[
                 "move",
                 "--debug-dump-rebase-plan",
-                "-s",
+                "--merge",
+                "--source",
                 &other_oid.to_string(),
             ],
             &GitRunOptions {
@@ -928,7 +947,7 @@ fn test_move_branches_after_move() -> eyre::Result<()> {
 
     // --on-disk
     {
-        let git = git.clone_repo()?;
+        let git = git.duplicate_repo()?;
 
         {
             let (stdout, stderr) = git.run(&[
@@ -1088,7 +1107,7 @@ fn test_move_no_reapply_upstream_commits() -> eyre::Result<()> {
 
     // --on-disk
     {
-        let git = git.clone_repo()?;
+        let git = git.duplicate_repo()?;
 
         {
             let (stdout, stderr) = git.run(&["move", "--on-disk", "-b", "HEAD", "-d", "master"])?;
@@ -1179,7 +1198,7 @@ fn test_move_no_reapply_squashed_commits() -> eyre::Result<()> {
 
     // --on-disk
     {
-        let git = git.clone_repo()?;
+        let git = git.duplicate_repo()?;
 
         {
             let (stdout, _stderr) = git.run(&["smartlog"])?;
@@ -1325,7 +1344,7 @@ fn test_move_delete_checked_out_branch() -> eyre::Result<()> {
 
     // --on-disk
     {
-        let git = git.clone_repo()?;
+        let git = git.duplicate_repo()?;
 
         {
             git.run(&["checkout", "work"])?;
@@ -1474,7 +1493,7 @@ fn test_move_merge_commit() -> eyre::Result<()> {
 
     // --on-disk
     {
-        let git = git.clone_repo()?;
+        let git = git.duplicate_repo()?;
 
         {
             let (stdout, stderr) = git.run(&[
@@ -1582,7 +1601,7 @@ fn test_move_merge_commit() -> eyre::Result<()> {
 
     // --in-memory
     {
-        let git = git.clone_repo()?;
+        let git = git.duplicate_repo()?;
 
         {
             let (stdout, _stderr) = git.run_with_options(
@@ -1701,7 +1720,7 @@ fn test_move_orphaned_root() -> eyre::Result<()> {
 
     // --on-disk
     {
-        let git = git.clone_repo()?;
+        let git = git.duplicate_repo()?;
 
         {
             let (stdout, stderr) = git.run(&["move", "--on-disk", "-d", "master"])?;
@@ -1809,3 +1828,45 @@ fn test_move_no_extra_checkout() -> eyre::Result<()> {
 
     Ok(())
 }
+
+#[test]
+fn test_move_dest_not_in_dag() -> eyre::Result<()> {
+    let GitWrapperWithRemoteRepo {
+        temp_dir: _guard,
+        original_repo,
+        cloned_repo,
+    } = make_git_with_remote_repo()?;
+
+    {
+        original_repo.init_repo()?;
+        original_repo.commit_file("test1", 1)?;
+        original_repo.commit_file("test2", 2)?;
+        original_repo.run(&["checkout", "-b", "other-branch", "HEAD^"])?;
+        original_repo.commit_file("test3", 3)?;
+
+        original_repo.clone_repo_into(&cloned_repo, &["--branch", "other-branch"])?;
+    }
+
+    {
+        cloned_repo.init_repo_with_options(&GitInitOptions {
+            make_initial_commit: false,
+            run_branchless_init: false,
+            ..Default::default()
+        })?;
+        cloned_repo.run(&["branchless", "init", "--main-branch", "other-branch"])?;
+        cloned_repo.run(&["config", "branchless.restack.preserveTimestamps", "true"])?;
+
+        let (stdout, _stderr) = cloned_repo.run(&["move", "-d", "origin/master"])?;
+        insta::assert_snapshot!(stdout, @r###"
+        Attempting rebase in-memory...
+        [1/1] Committed as: 70deb1e2 create test3.txt
+        branchless: processing 2 updates: branch other-branch, remote branch origin/other-branch
+        branchless: processing 1 rewritten commit
+        branchless: running command: <git-executable> checkout other-branch
+        Your branch is up to date with 'origin/other-branch'.
+        In-memory rebase succeeded.
+        "###);
+    }
+
+    Ok(())
+}
