@@ -8,8 +8,7 @@ use crate::util::trim_lines;
 use branchless::commands::undo::testing::{select_past_event, undo_events};
 use branchless::core::eventlog::{EventCursor, EventLogDb, EventReplayer};
 use branchless::core::formatting::Glyphs;
-use branchless::core::mergebase::make_merge_base_db;
-use branchless::git::{GitRunInfo, Repo};
+use branchless::git::{Dag, GitRunInfo, Repo};
 use branchless::testing::{make_git, Git};
 use branchless::tui::testing::{screen_to_string, CursiveTestingBackend, CursiveTestingEvent};
 use branchless::tui::Effects;
@@ -24,20 +23,22 @@ fn run_select_past_event(
 ) -> eyre::Result<Option<EventCursor>> {
     let glyphs = Glyphs::text();
     let effects = Effects::new_suppress_for_test(glyphs);
+    let references_snapshot = repo.get_references_snapshot()?;
     let conn = repo.get_db_conn()?;
     let event_log_db: EventLogDb = EventLogDb::new(&conn)?;
     let mut event_replayer = EventReplayer::from_event_log_db(&effects, &repo, &event_log_db)?;
-    let merge_base_db = make_merge_base_db(&effects, repo, &conn, &event_replayer)?;
+    let event_cursor = event_replayer.make_default_cursor();
+    let dag = Dag::open_and_sync(
+        &effects,
+        repo,
+        &event_replayer,
+        event_cursor,
+        &references_snapshot,
+    )?;
     let siv = CursiveRunnable::new::<Infallible, _>(move || {
         Ok(CursiveTestingBackend::init(events.clone()))
     });
-    select_past_event(
-        siv.into_runner(),
-        &effects,
-        repo,
-        &merge_base_db,
-        &mut event_replayer,
-    )
+    select_past_event(siv.into_runner(), &effects, repo, &dag, &mut event_replayer)
 }
 
 fn run_undo_events(git: &Git, event_cursor: EventCursor) -> eyre::Result<String> {
